@@ -434,6 +434,7 @@ class Backend(GridObjects, ABC):
         start_grid = self._grid
         self._grid = None
         res = copy.deepcopy(self)
+        res.__class__ = type(self)  # somehow deepcopy forget the init class... weird
         res._grid = copy.deepcopy(start_grid)
         self._grid = start_grid
         res._is_loaded = False  # i can reload a copy of an environment
@@ -758,9 +759,10 @@ class Backend(GridObjects, ABC):
         :type id_: int
 
         """
-        action = self._complete_action_class()
+        my_cls = type(self)
+        action = my_cls._complete_action_class()
         action.update({"set_line_status": [(id_, -1)]})
-        bk_act = self.my_bk_act_class()
+        bk_act = my_cls.my_bk_act_class()
         bk_act += action
         self.apply_action(bk_act)
 
@@ -1450,7 +1452,6 @@ class Backend(GridObjects, ABC):
                 q_s[bus_s == -1] = np.NaN
                 dict_["shunt"]["shunt_p"] = p_s
                 dict_["shunt"]["shunt_q"] = q_s
-
         set_me.update(dict_)
         return set_me
 
@@ -1532,10 +1533,11 @@ class Backend(GridObjects, ABC):
             # class is already initialized
             # and set up the proper class and everything
             self._init_class_attr()
+
+            # type(self)._INIT_GRID_CLS = orig_type
             # hack due to changing class of imported module in the module itself
             self.__class__ = type(self).init_grid(type(self), force_module=type(self).__module__)
             setattr(sys.modules[type(self).__module__], self.__class__.__name__, self.__class__)
-
             # reset the attribute of the grid2op.Backend.Backend class
             # that can be messed up with depending on the initialization of the backend
             Backend._clear_class_attribute()
@@ -1544,6 +1546,8 @@ class Backend(GridObjects, ABC):
         my_cls = type(self)
         my_cls.my_bk_act_class = _BackendAction.init_grid(my_cls)
         my_cls._complete_action_class = CompleteAction.init_grid(my_cls)
+        my_cls._complete_action_class._add_shunt_data()
+        my_cls._complete_action_class._update_value_set()
         my_cls.assert_grid_correct_cls()
 
     def assert_grid_correct_after_powerflow(self):
